/* ========================================
 *
 * Copyright Jerald M. Cogswell (C)2014
 * All Rights Reserved
 *
 * ========================================
*/
#include <project.h>
#include <DMA_1_dma.h>
#include <stdlib.h>
#include <math.h>
#include <stdio.h>
#include <math.h>
#if defined (__GNUC__)
    /* Add an explicit reference to the floating point printf library */
    /* to allow the usage of floating point conversion specifiers. */
    /* This is not linked in by default with the newlib-nano library. */
    asm (".global _printf_float");
#endif


/* *************        GLOBAL VARIABLES     **************** */
uint8 DMA_1_Chan;
uint8 DMA_ch_A;
uint8 DMA_ch_B;
// uint8 DMA_1_TD[2];
int16 Filter_Ch_A[2040]; // Holds output from Filter Channel A
int16 Filter_Ch_B[2040]; // Holds output from Filter Channel B
int16 High_IChA[5];
int16 High_IChA[5];
int16 High_IChB[5];
volatile uint8 switch_flag = 0, isr_dma_ADone_flag = 0, isr_dma_BDone_flag = 0;



/* *********************************************************** */
/* ***              FUNCTION DEFINITIONS                   *** */
/* *********************************************************** */

int16 avg_calc(int16[]);
int16 find_peak(int16[], int16[]);


/* ************************************************* *
**     Function to find average value in an array    *
**                                                   */
int16 avg_calc(int16 arry[])
    {
        int16 avg_val = 0;
        int16 hold16 = 0; // just a 16-bit holding value for interim calculations
        int16 i;
        for (i = 0; i < 5; i++) // size is the size of the array
        hold16 += arry[i];  // add all time delay values. We're doing a simple average here
        avg_val = hold16 * 0.2; // Same as divide by 5 but more efficient
        return avg_val;
    }
/* ******  End function to calculate average  ****** */
/* ------------------------------------------------- */
    
/* ************************************************* */
/* Funtion to find five highest peaks in data array  */
/*                                                   */
    int16 find_peak(int16 data[], int16 high[])
    {
    int16 previous[5];
    previous[0] = 0;  previous[1] = 0;  previous[2] = 0;  previous[3] = 0;  previous[4] = 0;  // quick initialize
    uint8 j;
    for (j = 0; j < 5; j++) // We have five segments of 408 samples
    {
        int16 i;
        uint16 k = j * 408;    // So k equals either 0 or 408 or 816 or 1224 or 1632
        for (i = 0; i < 408; i++)   // while i has values from 0 to 499
          {
            if (abs(data[i + k]) > previous[j])
                {
                    high[j] = i + k ;            // Keep highest peak position for each segment
                    previous[j] = data[i + k];   // and update previous peak to this new find
                }
          }
    }
    return 0;
    }
/*          End function to find peak values         */    
/* ------------------------------------------------- */

    // void DMA_Config(void);

/* *************   END FUNCTION DEFINITIONS   *************** */
      

int main()
{
    /* Place your initialization/startup code here (e.g. MyInst_Start()) */
    
int16 avg_calc(int16 arry[]); // declare array
void I2C_LCD_Position(uint8,uint8);
char8 I2C_LCD_PrString(int16);

/* ********************************** */
/* Variable declarations for DMA_1    */
/* ********************************** */  
    
/* Move these variable declarations to the top of the function */
uint8 DMA_1_Chan;
uint8 DMA_1_TD[2];

/* DMA Configuration for DMA_1 */
#define DMA_1_BYTES_PER_BURST 2
#define DMA_1_REQUEST_PER_BURST 1
#define DMA_1_SRC_BASE (CYDEV_PERIPH_BASE)
#define DMA_1_DST_BASE (CYDEV_PERIPH_BASE)
DMA_1_Chan = DMA_1_DmaInitialize(DMA_1_BYTES_PER_BURST, DMA_1_REQUEST_PER_BURST, 
    HI16(DMA_1_SRC_BASE), HI16(DMA_1_DST_BASE));
DMA_1_TD[0] = CyDmaTdAllocate();
DMA_1_TD[1] = CyDmaTdAllocate();
CyDmaTdSetConfiguration(DMA_1_TD[0], 2, DMA_1_TD[1], DMA_1__TD_TERMOUT_EN | TD_INC_SRC_ADR | TD_INC_DST_ADR | TD_AUTO_EXEC_NEXT);
CyDmaTdSetConfiguration(DMA_1_TD[1], 2, DMA_1_TD[0], DMA_1__TD_TERMOUT_EN | TD_INC_SRC_ADR | TD_INC_DST_ADR | TD_AUTO_EXEC_NEXT);
CyDmaTdSetAddress(DMA_1_TD[0], LO16((uint32)ADC_SAR_Seq_SAR_SAR_WRK0_PTR), LO16((uint32)Filter_STAGEA_PTR));
CyDmaTdSetAddress(DMA_1_TD[1], LO16((uint32)ADC_SAR_Seq_SAR_SAR_WRK0_PTR), LO16((uint32)Filter_STAGEB_PTR));
CyDmaChSetInitialTd(DMA_1_Chan, DMA_1_TD[0]);
CyDmaChEnable(DMA_1_Chan, 1);


/* ********************************** */
/* Variable declarations for DMA_ch_A */
/* ********************************** */

/* Move these variable declarations to the top of the function */
uint8 DMA_ch_A_Chan;
uint8 DMA_ch_A_TD[1];

/* DMA Configuration for DMA_ch_A */
#define DMA_ch_A_BYTES_PER_BURST 2
#define DMA_ch_A_REQUEST_PER_BURST 1
#define DMA_ch_A_SRC_BASE (CYDEV_PERIPH_BASE)
#define DMA_ch_A_DST_BASE (CYDEV_SRAM_BASE)
DMA_ch_A_Chan = DMA_ch_A_DmaInitialize(DMA_ch_A_BYTES_PER_BURST, DMA_ch_A_REQUEST_PER_BURST, 
    HI16(DMA_ch_A_SRC_BASE), HI16(DMA_ch_A_DST_BASE));
DMA_ch_A_TD[0] = CyDmaTdAllocate();
CyDmaTdSetConfiguration(DMA_ch_A_TD[0], 4080, CY_DMA_DISABLE_TD, DMA_ch_A__TD_TERMOUT_EN | TD_INC_SRC_ADR | TD_INC_DST_ADR);
CyDmaTdSetAddress(DMA_ch_A_TD[0], LO16((uint32)Filter_HOLDA_PTR), LO16((uint32)Filter_Ch_A));
CyDmaChSetInitialTd(DMA_ch_A_Chan, DMA_ch_A_TD[0]);
CyDmaChEnable(DMA_ch_A_Chan, 1);

/* End variable declarations for DMA_ch_A */


/* ********************************** */
/* Variable declarations for DMA_ch_B */
/* ********************************** */

/* Move these variable declarations to the top of the function */
uint8 DMA_ch_B_Chan;
uint8 DMA_ch_B_TD[1];

/* DMA Configuration for DMA_ch_B */
#define DMA_ch_B_BYTES_PER_BURST 2
#define DMA_ch_B_REQUEST_PER_BURST 1
#define DMA_ch_B_SRC_BASE (CYDEV_PERIPH_BASE)
#define DMA_ch_B_DST_BASE (CYDEV_SRAM_BASE)
DMA_ch_B_Chan = DMA_ch_B_DmaInitialize(DMA_ch_B_BYTES_PER_BURST, DMA_ch_B_REQUEST_PER_BURST, 
    HI16(DMA_ch_B_SRC_BASE), HI16(DMA_ch_B_DST_BASE));
DMA_ch_B_TD[0] = CyDmaTdAllocate();
CyDmaTdSetConfiguration(DMA_ch_B_TD[0], 4080, CY_DMA_DISABLE_TD, DMA_ch_B__TD_TERMOUT_EN | TD_INC_SRC_ADR | TD_INC_DST_ADR);
CyDmaTdSetAddress(DMA_ch_B_TD[0], LO16((uint32)Filter_HOLDB_PTR), LO16((uint32)Filter_Ch_B));
CyDmaChSetInitialTd(DMA_ch_B_Chan, DMA_ch_B_TD[0]);
CyDmaChEnable(DMA_ch_B_Chan, 1);

/* End variable declarations for DMA_ch_B */

        uint16 i;
        int16 High_IChA[5]; /* Holds 5 peak positions (not values) in the hold_SAR_A array */
        int16 High_IChB[5]; /* Holds 5 peak positions (not values) in the hold_SAR_B array */
        int16 sample_delay[5];
        int16 avg_delay;
        

/* **************** */

    /* End of initialization/startup code */
    

CyGlobalIntEnable; /* Uncomment this line to enable global interrupts.
    for (;;)
    {
    } */
        /* Place your application code here. */

        
/* Start the Filter */
void Filter_Start();

/* Start the ADC */
ADC_SAR_Seq_Start();
I2C_M_Start();
I2C_M_EnableInt();
I2C_M_Enable();
void I2C_LCD_Start(void);
I2C_LCD_DisplayOn(); // Is this needed? See Datasheet ************


/* ***********************************************************************
*  This section reads the 2040 samples of filtered data for each channel and divides it into 5 segments of 408 samples.
*  For each segment it finds the offset to the highest peak value and keeps those five peak positions (offsets).
*  We do this for Channel A and for Channel B.
*/
/* *** Find the five highest peaks in the Filter Channel A data *** */
    find_peak(Filter_Ch_A, High_IChA); // High_IChA array now contains the five pointers to highest peaks

/* *** Now do this for Channel B *** */
    find_peak(Filter_Ch_B, High_IChB); // High_IChA array now contains the five pointers to highest peaks
    
/* ***   Now we will find the offset (time delay) between peaks in Channel A and Channel B *** */
    for (i = 0; i < 5; i++)  sample_delay[i] = High_IChA[i] - High_IChB[i];
/* ***   Calculate simple average value of the delays *** */
    // We are dealing with positive and negative values so let's NOT do a root mean square.
    // We might consider rejecting outliers but not right now.
    avg_delay = avg_calc(sample_delay); // and take the average value of time delay
    // Print the difference on the LCD
    I2C_LCD_Position(0,0);
    I2C_LCD_PrintNumber(avg_delay);

    return 0;   
} /* end main */


/* [] END OF FILE */
